@@ -4,6 +4,7 @@ from optparse import make_option
 from django.core.management.base import NoArgsCommand
 from mapit.models import Generation
 
+
 class Command(NoArgsCommand):
     help = 'Actives the inactive generation'
     option_list = NoArgsCommand.option_list + (
@@ -13,11 +14,11 @@ class Command(NoArgsCommand):
     def handle(self, **options):
         new = Generation.objects.new()
         if not new:
-            raise Exception, "You do not have an inactive generation to activate"
+            raise Exception("You do not have an inactive generation to activate")
 
         new.active = True
         if options['commit']:
             new.save()
-            print "%s - activated" % new
+            print("%s - activated" % new)
         else:
-            print "%s - not activated, dry run" % new
+            print("%s - not activated, dry run" % new)
