@@ -5,20 +5,25 @@ from django.contrib.gis.db import models
 from django.contrib.gis.gdal import SRSException, OGRException
 from django.conf import settings
 from django.db import connection
+from django.db.models.query import RawQuerySet
 from django.utils.html import escape
+from django.utils.encoding import python_2_unicode_compatible
 
 from mapit.managers import Manager, GeoManager
 from mapit import countries
-from mapit.djangopatch import NoValidateRawQuerySet
+from mapit.djangopatch import GetQuerySetMetaclass
+from django.utils import six
 
-class GenerationManager(models.Manager):
+
+class GenerationManager(six.with_metaclass(GetQuerySetMetaclass, models.Manager)):
     def current(self):
         """Return the most recent active generation.
 
         If there are no active generations, return 0."""
 
-        latest_on = self.get_query_set().filter(active=True).order_by('-id')
-        if latest_on: return latest_on[0]
+        latest_on = self.get_queryset().filter(active=True).order_by('-id')
+        if latest_on:
+            return latest_on[0]
         return 0
 
     def new(self):
@@ -27,11 +32,13 @@ class GenerationManager(models.Manager):
         If there are no generations, or the most recent one is active,
         return None."""
 
-        latest = self.get_query_set().order_by('-id')
+        latest = self.get_queryset().order_by('-id')
         if not latest or latest[0].active:
             return None
         return latest[0]
-        
+
+
+@python_2_unicode_compatible
 class Generation(models.Model):
 
     # Generations are used so that, theoretically, old versions of the same
@@ -41,17 +48,17 @@ class Generation(models.Model):
     # lookups (both can be overridden to a different generation with a query
     # parameter). Inactive generations are so that you can load in new data
     # without it being returned by normal lookups by everyone using mapit.
-    # 
+    #
     # An Area in the database has a minimum and maximum generation that it is
     # valid for, so that you can see at which point an area was added and then
     # removed.
-    # 
+    #
     # As an example, http://mapit.mysociety.org/postcode/EH11BB.html is the
     # current areas for that postcode, whilst
     # http://mapit.mysociety.org/postcode/EH11BB.html?generation=14 gives you
     # the areas before the last Scottish Parliament boundary changes, hence
     # giving you the different areas involved.
-    # 
+    #
     # The concept works okay for boundary changes of things that have the
     # notion of being children - e.g. council wards, UK Parliament
     # constituencies, and so on - which are changed with a clean slate to a
@@ -63,7 +70,7 @@ class Generation(models.Model):
     # of Edinburgh Council boundary. If the City of Edinburgh Council boundary
     # were to change, this should get a new ID starting at the new generation.
     # But that would break some things.
-    # 
+    #
     # Another example, as I've just fixed #32, is
     # http://mapit.mysociety.org/area/2253/children.html?type=UTW vs
     # http://mapit.mysociety.org/area/2253/children.html?generation=14;type=UTW
@@ -71,11 +78,12 @@ class Generation(models.Model):
 
     active = models.BooleanField(default=False)
     created = models.DateTimeField(auto_now_add=True)
-    description = models.CharField(max_length=255, help_text="Describe this generation, eg '2010 electoral boundaries'")
+    description = models.CharField(
+        max_length=255, help_text="Describe this generation, eg '2010 electoral boundaries'")
 
     objects = GenerationManager()
 
-    def __unicode__(self):
+    def __str__(self):
         id = self.id or '?'
         return "Generation %s (%sactive)" % (id, "" if self.active else "in")
 
@@ -87,16 +95,20 @@ class Generation(models.Model):
             'description': self.description,
         }
 
+
+@python_2_unicode_compatible
 class Country(models.Model):
     code = models.CharField(max_length=3, unique=True)
     name = models.CharField(max_length=100, unique=True)
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name
-    
-    class Meta:
-        verbose_name_plural='countries'
 
+    class Meta:
+        verbose_name_plural = 'countries'
+
+
+@python_2_unicode_compatible
 class Type(models.Model):
 
     # An area type (the Type model) is the type of area. You can see examples
@@ -107,25 +119,30 @@ class Type(models.Model):
     # source data we used from Ordnance Survey.
 
     code = models.CharField(max_length=500, unique=True, help_text="A unique code, eg 'CTR', 'CON', etc")
-    description = models.CharField(max_length=200, blank=True, help_text="The name of the type of area, eg 'Country', 'Constituency', etc")
+    description = models.CharField(
+        max_length=200, blank=True, help_text="The name of the type of area, eg 'Country', 'Constituency', etc")
 
-    def __unicode__(self):
+    def __str__(self):
         return '%s (%s)' % (self.description, self.code)
 
-class AreaManager(models.GeoManager):
-    def get_query_set(self):
-        return super(AreaManager, self).get_query_set().select_related('type', 'country')
+
+class AreaManager(six.with_metaclass(GetQuerySetMetaclass, models.GeoManager)):
+    def get_queryset(self):
+        return super(AreaManager, self).get_queryset().select_related('type', 'country')
 
     def by_location(self, location, generation=None):
-        if generation is None: generation = Generation.objects.current()
-        if not location: return []
+        if generation is None:
+            generation = Generation.objects.current()
+        if not location:
+            return []
         return Area.objects.filter(
             polygons__polygon__contains=location,
             generation_low__lte=generation, generation_high__gte=generation
         )
 
     def by_postcode(self, postcode, generation=None):
-        if not generation: generation = Generation.objects.current()
+        if not generation:
+            generation = Generation.objects.current()
         return list(itertools.chain(
             self.by_location(postcode.location, generation),
             postcode.areas.filter(
@@ -137,17 +154,18 @@ class AreaManager(models.GeoManager):
     # We force the non-geographical part of the query to be done first, because
     # if a type is specified, that greatly speeds it up.
     def intersect(self, query_type, area, types, generation):
-        if not isinstance(query_type, list): query_type = [ query_type ]
+        if not isinstance(query_type, list):
+            query_type = [query_type]
 
-        params = [ area.id, area.id, generation.id, generation.id ]
+        params = [area.id, area.id, generation.id, generation.id]
 
         if types:
-            params.append( tuple(types) )
+            params.append(tuple(types))
             query_area_type = ' AND mapit_area.type_id IN (SELECT id FROM mapit_type WHERE code IN %s) '
         else:
             query_area_type = ''
 
-        query_geo = ' OR '.join([ 'ST_%s(geometry.polygon, target.polygon)' % type for type in query_type ])
+        query_geo = ' OR '.join(['ST_%s(geometry.polygon, target.polygon)' % type for type in query_type])
 
         query = '''
 WITH
@@ -166,16 +184,16 @@ SELECT DISTINCT mapit_area.*
   FROM mapit_area, geometry, target
  WHERE geometry.area_id = mapit_area.id AND (%s)
 ''' % (query_area_type, query_geo)
-        # Monkeypatched self.raw() here to prevent needless SQL validation (removed from Django 1.3)
-        return NoValidateRawQuerySet(raw_query=query, model=self.model, params=params, using=self._db)
+        return RawQuerySet(raw_query=query, model=self.model, params=params, using=self._db)
 
     def get_or_create_with_name(self, country=None, type=None, name_type='', name=''):
         current_generation = Generation.objects.current()
         new_generation = Generation.objects.new()
-        area, created = Area.objects.get_or_create(country=country, type=type,
+        area, created = Area.objects.get_or_create(
+            country=country, type=type,
             generation_low__lte=current_generation, generation_high__gte=current_generation,
             names__type__code=name_type, names__name=name,
-            defaults = { 'generation_low': new_generation, 'generation_high': new_generation }
+            defaults={'generation_low': new_generation, 'generation_high': new_generation}
         )
         if created:
             area.names.get_or_create(type=NameType.objects.get(code=name_type), name=name)
@@ -187,10 +205,11 @@ SELECT DISTINCT mapit_area.*
     def get_or_create_with_code(self, country=None, type=None, code_type='', code=''):
         current_generation = Generation.objects.current()
         new_generation = Generation.objects.new()
-        area, created = Area.objects.get_or_create(country=country, type=type,
+        area, created = Area.objects.get_or_create(
+            country=country, type=type,
             generation_low__lte=current_generation, generation_high__gte=current_generation,
             codes__type__code=code_type, codes__code=code,
-            defaults = { 'generation_low': new_generation, 'generation_high': new_generation }
+            defaults={'generation_low': new_generation, 'generation_high': new_generation}
         )
         if created:
             area.codes.get_or_create(type=CodeType.objects.get(code=code_type), code=code)
@@ -199,11 +218,14 @@ SELECT DISTINCT mapit_area.*
             area.save()
         return area
 
+
 class TransformError(Exception):
     pass
 
+
+@python_2_unicode_compatible
 class Area(models.Model):
-    name = models.CharField(max_length=2000, editable=False, blank=True) # Automatically set from name children
+    name = models.CharField(max_length=2000, blank=True)
     parent_area = models.ForeignKey('self', related_name='children', null=True, blank=True)
     type = models.ForeignKey(Type, related_name='areas')
     country = models.ForeignKey(Country, related_name='areas', null=True, blank=True)
@@ -224,7 +246,7 @@ class Area(models.Model):
             codes[code.type.code] = code.code
         return codes
 
-    def __unicode__(self):
+    def __str__(self):
         name = self.name or '(unknown)'
         return '%s %s' % (self.type.code, name)
 
@@ -301,15 +323,16 @@ class Area(models.Model):
             try:
                 all_areas.transform(srid)
             except (SRSException, OGRException) as e:
-                raise TransformError, "Error with transform: %s" % e
+                raise TransformError("Error with transform: %s" % e)
 
         num_points_before_simplification = all_areas.num_points
         if simplify_tolerance:
             all_areas = all_areas.simplify(simplify_tolerance)
             if all_areas.num_points == 0 and num_points_before_simplification > 0:
-                raise TransformError, "Simplifying %s with tolerance %f left no boundary at all" % (self, simplify_tolerance)
+                raise TransformError("Simplifying %s with tolerance %f left no boundary at all" % (
+                    self, simplify_tolerance))
 
-        if export_format=='kml':
+        if export_format == 'kml':
             if kml_type == "polygon":
                 out = all_areas.kml
             elif kml_type == "full":
@@ -333,16 +356,18 @@ class Area(models.Model):
     </Document>
 </kml>''' % (line_colour, fill_colour, escape(self.name), all_areas.kml)
             else:
-                raise Exception, "Unknown kml_type: '%s'" % (kml_type,)
+                raise Exception("Unknown kml_type: '%s'" % (kml_type,))
             content_type = 'application/vnd.google-earth.kml+xml'
         elif export_format in ('json', 'geojson'):
             out = all_areas.json
             content_type = 'application/json'
-        elif export_format=='wkt':
+        elif export_format == 'wkt':
             out = all_areas.wkt
             content_type = 'text/plain'
         return (out, content_type)
 
+
+@python_2_unicode_compatible
 class Geometry(models.Model):
     area = models.ForeignKey(Area, related_name='polygons')
     polygon = models.PolygonField(srid=settings.MAPIT_AREA_SRID)
@@ -351,9 +376,11 @@ class Geometry(models.Model):
     class Meta:
         verbose_name_plural = 'geometries'
 
-    def __unicode__(self):
-        return u'%s, polygon %d' % (self.area, self.id)
+    def __str__(self):
+        return '%s, polygon %d' % (self.area, self.id)
 
+
+@python_2_unicode_compatible
 class NameType(models.Model):
 
     # Name types are for storing different types of names. This could have
@@ -362,13 +389,17 @@ class NameType(models.Model):
     # itself; in global MaPit, the different language names are stored here
     # and displayed in the alternative names section.
 
-    code = models.CharField(max_length=10, unique=True, help_text="A unique code to identify this type of name: eg 'english' or 'iso'")
-    description = models.CharField(max_length=200, blank=True, help_text="The name of this type of name, eg 'English' or 'ISO Standard'")
+    code = models.CharField(
+        max_length=10, unique=True, help_text="A unique code to identify this type of name: eg 'english' or 'iso'")
+    description = models.CharField(
+        max_length=200, blank=True, help_text="The name of this type of name, eg 'English' or 'ISO Standard'")
     objects = Manager()
 
-    def __unicode__(self):
+    def __str__(self):
         return '%s (%s)' % (self.description, self.code)
 
+
+@python_2_unicode_compatible
 class Name(models.Model):
     area = models.ForeignKey(Area, related_name='names')
     type = models.ForeignKey(NameType, related_name='names')
@@ -378,45 +409,19 @@ class Name(models.Model):
     class Meta:
         unique_together = ('area', 'type')
 
-    def __unicode__(self):
+    def __str__(self):
         return '%s (%s) [%s]' % (self.name, self.type.code, self.area.id)
 
-    def make_friendly_name(self, name):
-        n = re.sub('\s+', ' ', name.name.strip())
-        n = n.replace('St. ', 'St ')
-        if name.type.code == 'M': return n
-        if name.type.code == 'S': return n
-        # Type must be 'O' here
-        n = re.sub(' Euro Region$', '', n) # EUR
-        n = re.sub(' (Burgh|Co|Boro) Const$', '', n) # WMC
-        n = re.sub(' P Const$', '', n) # SPC
-        n = re.sub(' PER$', '', n) # SPE
-        n = re.sub(' GL Assembly Const$', '', n) # LAC
-        n = re.sub(' Assembly Const$', '', n) # WAC
-        n = re.sub(' Assembly ER$', '', n) # WAE
-        n = re.sub(' London Boro$', ' Borough', n) # LBO
-        if self.area.country and self.area.country.name == 'Wales': n = re.sub('^.*? - ', '', n) # UTA
-        n = re.sub('(?:The )?City of (.*?) (District )?\(B\)$', r'\1 City', n) # UTA
-        n = re.sub(' District \(B\)$', ' Borough', n) # DIS
-        n = re.sub(' \(B\)$', ' Borough', n) # DIS
-        if self.area.type.code in ('CTY', 'DIS', 'LBO', 'UTA', 'MTD'): n += ' Council'
-        n = re.sub(' (ED|CP)$', '', n) # CPC, CED, UTE
-        n = re.sub(' Ward$', '', n) # DIW, LBW, MTW, UTW
-        return n
-
     def save(self, *args, **kwargs):
         super(Name, self).save(*args, **kwargs)
-        try:
-            name = self.area.names.filter(type__code__in=('M', 'O', 'S')).order_by('type__code')[0]
-            self.area.name = self.make_friendly_name(name)
-            self.area.save()
-        except:
-            pass
+        if hasattr(countries, 'name_save_hook'):
+            countries.name_save_hook(self)
 
     def as_tuple(self):
-        return (self.type.code, [self.type.description,
-                                 self.name])
+        return (self.type.code, [self.type.description, self.name])
 
+
+@python_2_unicode_compatible
 class CodeType(models.Model):
 
     # Code types are so you can store different types of code for an area. In
@@ -426,11 +431,15 @@ class CodeType(models.Model):
     # object, perhaps.
 
     code = models.CharField(max_length=10, unique=True, help_text="A unique code, eg 'ons' or 'unit_id'")
-    description = models.CharField(max_length=200, blank=True, help_text="The name of the code, eg 'Office of National Statitics' or 'Ordnance Survey ID'")
+    description = models.CharField(
+        max_length=200, blank=True,
+        help_text="The name of the code, eg 'Office of National Statitics' or 'Ordnance Survey ID'")
 
-    def __unicode__(self):
+    def __str__(self):
         return '%s (%s)' % (self.description, self.code)
 
+
+@python_2_unicode_compatible
 class Code(models.Model):
     area = models.ForeignKey(Area, related_name='codes')
     type = models.ForeignKey(CodeType, related_name='codes')
@@ -440,17 +449,21 @@ class Code(models.Model):
     class Meta:
         unique_together = ('area', 'type')
 
-    def __unicode__(self):
+    def __str__(self):
         return '%s (%s) [%s]' % (self.code, self.type.code, self.area.id)
 
+
 # Postcodes
 
-class PostcodeManager(GeoManager):
-    def get_query_set(self):
+class PostcodeManager(six.with_metaclass(GetQuerySetMetaclass, GeoManager)):
+    def get_queryset(self):
         return self.model.QuerySet(self.model)
-    def __getattr__(self, attr, *args):
-        return getattr(self.get_query_set(), attr, *args)
 
+    def __getattr__(self, attr, *args):
+        return getattr(self.get_queryset(), attr, *args)
+
+
+@python_2_unicode_compatible
 class Postcode(models.Model):
     postcode = models.CharField(max_length=7, db_index=True, unique=True)
     location = models.PointField(null=True)
@@ -467,16 +480,17 @@ class Postcode(models.Model):
         # Plus this way we can keep the polygons in the database
         # without pulling out in a giant WKB string
         def filter_by_area(self, area):
-            collect = 'ST_Transform((select ST_Collect(polygon) from mapit_geometry where area_id=%s group by area_id), 4326)'
+            collect = '''ST_Transform((select ST_Collect(polygon) from mapit_geometry
+                where area_id=%s group by area_id), 4326)'''
             return self.extra(
-                where = [
+                where=[
                     'location && %s' % collect,
                     'ST_CoveredBy(location, %s)' % collect
                 ],
-                params = [ area.id, area.id ]
+                params=[area.id, area.id]
             )
 
-    def __unicode__(self):
+    def __str__(self):
         return self.get_postcode_display()
 
     # Prettify postcode for display, if we know how to
@@ -504,8 +518,8 @@ class Postcode(models.Model):
     # The database has the right proj4 text, the proj file does not. I think.
     def as_irish_grid(self):
         cursor = connection.cursor()
-        cursor.execute("SELECT ST_AsText(ST_Transform(ST_GeomFromText('POINT(%f %f)', 4326), 29902))" % (self.location[0], self.location[1]))
+        cursor.execute("SELECT ST_AsText(ST_Transform(ST_GeomFromText('POINT(%f %f)', 4326), 29902))" % (
+            self.location[0], self.location[1]))
         row = cursor.fetchone()
         m = re.match('POINT\((.*?) (.*)\)', row[0])
-        return map(float, m.groups())
-
+        return list(map(float, m.groups()))
