@@ -1,20 +1,22 @@
+import imp
 import os
-import sys
 import yaml
+
 import django
 
 # Path to here is something like
-# /data/vhost/<vhost>/<repo>/<project_name>/settings.py
+# .../<repo>/<project_name>/settings.py
 PROJECT_DIR = os.path.abspath(os.path.dirname(__file__))
-PROJECT_ROOT = os.path.abspath(os.path.join(PROJECT_DIR, '..'))
-PARENT_DIR = os.path.abspath(os.path.join(PROJECT_ROOT, '..'))
+BASE_DIR = os.path.dirname(PROJECT_DIR)
+PARENT_DIR = os.path.dirname(BASE_DIR)
 
 # The mySociety deployment system works by having a conf directory at the root
 # of the repo, containing a general.yml file of options. Use that file if
 # present. Obviously you can just edit any part of this file, it is a normal
 # Django settings.py file.
 try:
-    config = yaml.load( open(os.path.join(PROJECT_ROOT, 'conf', 'general.yml'), 'r') )
+    with open(os.path.join(BASE_DIR, 'conf', 'general.yml'), 'r') as fp:
+        config = yaml.load(fp)
 except:
     config = {}
 
@@ -48,13 +50,17 @@ if DEBUG:
     }
     CACHE_MIDDLEWARE_SECONDS = 0
 else:
-    CACHES = {
-        'default': {
-            'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
-            'LOCATION': '127.0.0.1:11211',
-            'TIMEOUT': 86400,
+    try:
+        import memcache  # noqa
+        CACHES = {
+            'default': {
+                'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
+                'LOCATION': '127.0.0.1:11211',
+                'TIMEOUT': 86400,
+            }
         }
-    }
+    except ImportError:
+        pass
     CACHE_MIDDLEWARE_SECONDS = 86400
     CACHE_MIDDLEWARE_KEY_PREFIX = config.get('MAPIT_DB_NAME')
 
@@ -63,7 +69,6 @@ if config.get('BUGS_EMAIL'):
     ADMINS = (
         ('mySociety bugs', config['BUGS_EMAIL']),
     )
-    MANAGERS = ADMINS
 
 DATABASES = {
     'default': {
@@ -98,8 +103,6 @@ else:
     TIME_ZONE = 'Europe/London'
     LANGUAGE_CODE = 'en'
 
-SITE_ID = 1
-
 # If you set this to False, Django will make some optimizations so as not
 # to load the internationalization machinery.
 USE_I18N = True
@@ -124,7 +127,7 @@ MEDIA_URL = ''
 # Don't put anything in this directory yourself; store your static files
 # in apps' "static/" subdirectories and in STATICFILES_DIRS.
 # Example: "/var/www/example.com/static/"
-STATIC_ROOT = os.path.join( PARENT_DIR, 'collected_static' )
+STATIC_ROOT = os.path.join(PARENT_DIR, 'collected_static')
 
 # URL prefix for static files.
 # Example: "http://example.com/static/", "http://static.example.com/"
@@ -142,15 +145,13 @@ STATICFILES_DIRS = (
 STATICFILES_FINDERS = (
     'django.contrib.staticfiles.finders.FileSystemFinder',
     'django.contrib.staticfiles.finders.AppDirectoriesFinder',
-#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
+    # 'django.contrib.staticfiles.finders.DefaultStorageFinder',
 )
 
 # List of callables that know how to import templates from various sources.
 TEMPLATE_LOADERS = (
     'django.template.loaders.filesystem.Loader',
-    # Needs adapting to new class version
-    #'django.template.loaders.app_directories.Loader',
-    'mapit.loader.load_template_source',
+    'django.template.loaders.app_directories.Loader',
 )
 
 # UpdateCacheMiddleware does ETag setting, and
@@ -191,17 +192,25 @@ TEMPLATE_CONTEXT_PROCESSORS = (
     'mapit.context_processors.analytics',
 )
 
-INSTALLED_APPS = (
+INSTALLED_APPS = [
     'django.contrib.auth',
     'django.contrib.contenttypes',
     'django.contrib.messages',
     'django.contrib.sessions',
-    'django.contrib.sites',
     'django.contrib.admin',
     'django.contrib.gis',
     'django.contrib.staticfiles',
 
-    'south',
     'mapit',
-)
+]
+if django.VERSION < (1, 7):
+    INSTALLED_APPS.append('south')
 
+if MAPIT_COUNTRY:
+    try:
+        c = 'mapit_%s' % MAPIT_COUNTRY.lower()
+        imp.find_module(c)
+        # Put before 'mapit', so country templates take precedence
+        INSTALLED_APPS.insert(INSTALLED_APPS.index('mapit'), c)
+    except:
+        pass
