@@ -4,7 +4,8 @@ admin.autodiscover()
 
 handler500 = 'mapit.shortcuts.json_500'
 
-urlpatterns = patterns('',
+urlpatterns = patterns(
+    '',
     (r'^', include('mapit.urls')),
     (r'^admin/', include(admin.site.urls)),
 )
